// This file is part of GNOME Games. License: GPL-3.0+.

public interface Games.InputCapabilities : Object {
	public abstract bool get_allow_keyboard_mode ();
	public abstract bool get_allow_gamepad_mode ();
}
