// This file is part of GNOME Games. License: GPL-3.0+.

private errordomain Games.CueSheetError {
	UNEXPECTED_TOKEN,
	UNEXPECTED_EOL,
	UNEXPECTED_EOF,
	INVALID_FILE_FORMAT,
	INVALID_TRACK_NUMBER,
	INVALID_TRACK_MODE,
	NOT_A_TRACK,
}
